@@ -2,6 +2,7 @@ use super::context::*;
 use super::decode::*;
 use super::header::*;
 use super::lead::*;
+use super::prim::*;
 use super::state::*;
 use super::types::*;
 
@@ -49,7 +50,11 @@ pub enum ReaderError {
     /// Unexpected type received in an indefinite Bytes where only definite Bytes chunk are allowed
     WrongExpectedTypeInBytes { got: Type },
     /// Expected termination, but still some trailing data available
-    NotTerminated { at: usize, remaining_bytes: usize },
+    NotTerminated {
+        at: usize,
+        remaining_bytes: usize,
+        next_byte: u8,
+    },
 }
 
 impl From<LeadError> for ReaderError {
@@ -127,6 +132,7 @@ impl<'a> Reader<'a> {
             return Err(ReaderError::NotTerminated {
                 at: self.consumed_bytes(),
                 remaining_bytes: self.remaining_bytes(),
+                next_byte: self.reader.peek_byte(),
             });
         }
         Ok(())
@@ -417,6 +423,9 @@ impl<'a> Reader<'a> {
                     let data = self.cbor_slice_neutral()?;
                     elements.push(data);
                 }
+                // skip the break now that we found it
+                self.reader.advance(1);
+
                 Ok(Array {
                     len_encoding: content.into(),
                     elements,
@@ -448,12 +457,16 @@ impl<'a> Reader<'a> {
         match content {
             // indefinite Map
             None => {
-                // loop for cbor slices until we find a cbor break
+                // loop for cbor key/value slices until we find a cbor break
                 while self.peek_type()? != Type::Break {
                     let key = self.cbor_slice_neutral()?;
                     let value = self.cbor_slice_neutral()?;
                     elements.push((key, value));
                 }
+
+                // skip the break now that we found it
+                self.reader.advance(1);
+
                 Ok(Map {
                     len_encoding: content.into(),
                     elements,
@@ -520,10 +533,13 @@ impl<'a> Reader<'a> {
         }
     }
 
-    pub fn decodable_slice<T: Decode + 'static>(
-        &mut self,
-    ) -> Result<&'a CborSliceOf<T>, DecodeError> {
+    pub fn exact_decodable_slice<T: Decode>(&mut self) -> Result<&'a CborSliceOf<T>, DecodeError> {
         let slice = self.cbor_slice_neutral()?;
         slice.validate_as()
     }
+
+    pub fn exact_decodable_data<T: Decode>(&mut self) -> Result<CborDataOf<T>, DecodeError> {
+        let slice = self.cbor_slice_neutral()?;
+        slice.validate_as().map(|slice| slice.to_owned())
+    }
 }
